import { map, first, filter } from 'rxjs/operators';
import { Commands, Kinds } from '../configuration/xcWebSocketBridgeConfiguration';
import { SubscriberEventType } from '../configuration/apiConfiguration';
import 'rxjs/add/operator/toPromise';
import { getHeaderWithIncomingType, Serializer, Deserializer, fatalErrorState, } from './xcomponentMessages';
import { StateMachineInstance } from '../interfaces/StateMachineInstance';
import { generateUUID } from '../utils/uuid';
import { Logger } from '../utils/Logger';
var WebSocketSubscriber = /** @class */ (function () {
    function WebSocketSubscriber(webSocketWrapper, configuration) {
        this.webSocketWrapper = webSocketWrapper;
        this.configuration = configuration;
        this.logger = Logger.getLogger('WebSocketSubscriber');
        this.subscribedStateMachines = {};
        this.deserializer = new Deserializer();
        this.serializer = new Serializer();
        this.timeout = '00:00:10';
        var thisSubscriber = this;
        var webSocketUpdates$ = this.webSocketWrapper.getObservable();
        if (webSocketUpdates$ !== undefined && webSocketUpdates$ !== null) {
            this.updates$ = webSocketUpdates$.pipe(map(function (rawMessage) {
                return thisSubscriber.deserializer.deserialize(rawMessage.data || rawMessage);
            }));
        }
    }
    WebSocketSubscriber.prototype.setStateMachineRefSendPublisher = function (stateMachineRefSendPublisher) {
        this.stateMachineRefSendPublisher = stateMachineRefSendPublisher;
    };
    WebSocketSubscriber.prototype.getSnapshot = function (componentName, stateMachineName, privateTopics) {
        var replyTopic = generateUUID();
        var thisSubscriber = this;
        var promise = this.updates$
            .pipe(filter(function (data) {
            return data.command === Commands[Commands.snapshot] && data.topic === replyTopic;
        }), first(), map(function (data) {
            thisSubscriber.sendUnsubscribeRequestToTopic(replyTopic, Kinds.Snapshot);
            return thisSubscriber.getJsonDataFromSnapshot(data.stringData, data.topic);
        }))
            .toPromise();
        this.sendSubscribeRequestToTopic(replyTopic, Kinds.Snapshot);
        var dataToSendSnapshot = this.getDataToSendSnapshot(componentName, stateMachineName, replyTopic, privateTopics);
        this.webSocketWrapper.send(thisSubscriber.serializer.convertToWebsocketInputFormat(dataToSendSnapshot));
        return promise;
    };
    WebSocketSubscriber.prototype.getDataToSendSnapshot = function (componentName, stateMachineName, replyTopic, privateTopics) {
        var componentCode = this.configuration.getComponentCode(componentName);
        var stateMachineCode = this.configuration.getStateMachineCode(componentName, stateMachineName);
        var topic = this.configuration.getSnapshotTopic(componentCode);
        var jsonMessage = {
            Timeout: this.timeout,
            CallerPrivateTopic: privateTopics.getSubscriberTopics(),
            ReplyTopic: replyTopic,
        };
        var header = getHeaderWithIncomingType();
        header.ComponentCode = componentCode;
        header.StateMachineCode = stateMachineCode;
        var dataToSendSnapshot = {
            RoutingKey: topic,
            ComponentCode: componentCode,
            Event: {
                Header: header,
                JsonMessage: JSON.stringify(jsonMessage),
            },
        };
        return dataToSendSnapshot;
    };
    WebSocketSubscriber.prototype.prepareStateMachineUpdates = function (componentName, stateMachineName) {
        var componentCode = this.configuration.getComponentCode(componentName);
        var stateMachineCode = this.configuration.getStateMachineCode(componentName, stateMachineName);
        var thisSubscriber = this;
        var filteredObservable = this.updates$.pipe(filter(function (data) { return data.command === Commands[Commands.update]; }), map(function (data) { return thisSubscriber.getJsonDataFromEvent(data.stringData, data.topic); }), filter(function (jsonData) {
            return thisSubscriber.isSameComponent(jsonData, componentCode) &&
                thisSubscriber.isSameStateMachine(jsonData, stateMachineCode);
        }));
        return filteredObservable;
    };
    WebSocketSubscriber.prototype.isSameComponent = function (jsonData, componentCode) {
        var sameComponent = jsonData.stateMachineRef.ComponentCode === componentCode;
        return sameComponent;
    };
    WebSocketSubscriber.prototype.isSameStateMachine = function (jsonData, stateMachineCode) {
        var sameStateMachine = jsonData.stateMachineRef.StateMachineCode === stateMachineCode;
        return sameStateMachine;
    };
    WebSocketSubscriber.prototype.getStateMachineUpdates = function (componentName, stateMachineName) {
        var filteredObservable = this.prepareStateMachineUpdates(componentName, stateMachineName);
        this.sendSubscribeRequest(componentName, stateMachineName);
        return filteredObservable;
    };
    WebSocketSubscriber.prototype.canSubscribe = function (componentName, stateMachineName) {
        var componentCode = this.configuration.getComponentCode(componentName);
        var stateMachineCode = this.configuration.getStateMachineCode(componentName, stateMachineName);
        return this.configuration.containsSubscriber(componentCode, stateMachineCode, SubscriberEventType.Update);
    };
    WebSocketSubscriber.prototype.subscribe = function (componentName, stateMachineName, stateMachineUpdateListener) {
        this.prepareStateMachineUpdates(componentName, stateMachineName).subscribe(function (stateMachineInstance) {
            stateMachineUpdateListener.onStateMachineUpdate(stateMachineInstance);
        });
        this.sendSubscribeRequest(componentName, stateMachineName);
    };
    WebSocketSubscriber.prototype.sendSubscribeRequest = function (componentName, stateMachineName) {
        if (!this.isSubscribed(this.subscribedStateMachines, componentName, stateMachineName)) {
            var componentCode = this.configuration.getComponentCode(componentName);
            var stateMachineCode = this.configuration.getStateMachineCode(componentName, stateMachineName);
            var topic = this.configuration.getSubscriberTopic(componentCode, stateMachineCode, SubscriberEventType.Update);
            var kind = Kinds.Public;
            this.sendSubscribeRequestToTopic(topic, kind);
            this.addSubscribedStateMachines(componentName, stateMachineName);
        }
    };
    WebSocketSubscriber.prototype.sendSubscribeRequestToTopic = function (topic, kind) {
        var data = this.getDataToSend(topic, kind);
        var commandData = {
            Command: Commands[Commands.subscribe],
            Data: data,
        };
        var input = this.serializer.convertCommandDataToWebsocketInputFormat(commandData);
        this.webSocketWrapper.send(input);
    };
    WebSocketSubscriber.prototype.sendUnsubscribeRequestToTopic = function (topic, kind) {
        var data = this.getDataToSend(topic, kind);
        var commandData = {
            Command: Commands[Commands.unsubscribe],
            Data: data,
        };
        var input = this.serializer.convertCommandDataToWebsocketInputFormat(commandData);
        this.webSocketWrapper.send(input);
    };
    WebSocketSubscriber.prototype.getDataToSend = function (topic, kind) {
        return {
            Header: getHeaderWithIncomingType(),
            JsonMessage: JSON.stringify({
                Topic: {
                    Key: topic,
                    kind: kind,
                },
            }),
        };
    };
    WebSocketSubscriber.prototype.unsubscribe = function (componentName, stateMachineName) {
        if (this.isSubscribed(this.subscribedStateMachines, componentName, stateMachineName)) {
            var componentCode = this.configuration.getComponentCode(componentName);
            var stateMachineCode = this.configuration.getStateMachineCode(componentName, stateMachineName);
            var topic = this.configuration.getSubscriberTopic(componentCode, stateMachineCode, SubscriberEventType.Update);
            var kind = Kinds.Public;
            var data = this.getDataToSend(topic, kind);
            var commandData = {
                Command: Commands[Commands.unsubscribe],
                Data: data,
            };
            this.webSocketWrapper.send(this.serializer.convertCommandDataToWebsocketInputFormat(commandData));
            this.removeSubscribedStateMachines(componentName, stateMachineName);
        }
    };
    WebSocketSubscriber.prototype.isSubscribed = function (subscribedStateMachines, componentName, stateMachineName) {
        var isSubscribed = subscribedStateMachines[componentName] !== undefined &&
            subscribedStateMachines[componentName].indexOf(stateMachineName) > -1;
        return isSubscribed;
    };
    WebSocketSubscriber.prototype.addSubscribedStateMachines = function (componentName, stateMachineName) {
        if (this.subscribedStateMachines[componentName] === undefined) {
            this.subscribedStateMachines[componentName] = [stateMachineName];
        }
        else {
            this.subscribedStateMachines[componentName].push(stateMachineName);
        }
    };
    WebSocketSubscriber.prototype.removeSubscribedStateMachines = function (componentName, stateMachineName) {
        var index = this.subscribedStateMachines[componentName].indexOf(stateMachineName);
        this.subscribedStateMachines[componentName].splice(index, 1);
    };
    WebSocketSubscriber.prototype.getJsonDataFromSnapshot = function (data, topic) {
        this.logger.debug('JsonData received from snapshot: ', { data: data, topic: topic }, 2);
        var jsonData = this.deserializer.getJsonData(data);
        var b64Data = JSON.parse(jsonData.JsonMessage).Items;
        var items;
        try {
            var raw = JSON.parse(this.deserializer.decodeServerMessage(b64Data));
            if (raw.$values) {
                items = raw.$values;
            }
            else {
                items = raw;
            }
        }
        catch (e) {
            items = b64Data;
        }
        var snapshotItems = new Array();
        for (var i = 0; i < items.length; i++) {
            var stateMachineRef = this.getStateMachineRef(items[i].StateMachineId, parseInt(items[i].WorkerId, undefined), parseInt(items[i].ComponentCode, undefined), parseInt(items[i].StateMachineCode, undefined), parseInt(items[i].StateCode, undefined));
            snapshotItems.push(new StateMachineInstance(stateMachineRef, items[i].PublicMember));
        }
        return snapshotItems;
    };
    WebSocketSubscriber.prototype.getJsonDataFromEvent = function (data, topic) {
        this.logger.debug('JsonData received from event: ', { data: data, topic: topic }, 2);
        var jsonData = this.deserializer.getJsonData(data);
        var stateMachineRef = this.getStateMachineRef(jsonData.Header.StateMachineId, jsonData.Header.WorkerId, jsonData.Header.ComponentCode, jsonData.Header.StateMachineCode, jsonData.Header.StateCode, jsonData.Header.ErrorMessage);
        return new StateMachineInstance(stateMachineRef, JSON.parse(jsonData.JsonMessage));
    };
    WebSocketSubscriber.prototype.getStateMachineRef = function (StateMachineId, workerId, componentCode, stateMachineCode, stateCode, errorMessage) {
        var thisSubscriber = this;
        var stateMachineRef = {
            StateMachineId: StateMachineId,
            WorkerId: workerId,
            ComponentCode: componentCode,
            StateMachineCode: stateMachineCode,
            StateName: errorMessage
                ? fatalErrorState
                : thisSubscriber.configuration.getStateName(componentCode, stateMachineCode, stateCode),
            send: function (messageType, jsonMessage, visibilityPrivate, specifiedPrivateTopic) {
                thisSubscriber.stateMachineRefSendPublisher.sendWithStateMachineRef(stateMachineRef, messageType, jsonMessage, visibilityPrivate, specifiedPrivateTopic);
            },
            ErrorMessage: errorMessage,
        };
        return stateMachineRef;
    };
    return WebSocketSubscriber;
}());
export { WebSocketSubscriber };
//# sourceMappingURL=WebSocketSubscriber.js.map