import { normalizeCommunication } from './apiCommunicationUtils';
export var SubscriberEventType;
(function (SubscriberEventType) {
    SubscriberEventType[SubscriberEventType["Update"] = 0] = "Update";
    SubscriberEventType[SubscriberEventType["Error"] = 1] = "Error";
})(SubscriberEventType || (SubscriberEventType = {}));
var DefaultApiConfiguration = /** @class */ (function () {
    function DefaultApiConfiguration(rawConfig) {
        this._config = rawConfig;
        if (!this.validate()) {
            throw new Error('invalid configuration');
        }
    }
    DefaultApiConfiguration.prototype.validate = function () {
        return (this._config !== undefined &&
            this._config.deployment !== undefined &&
            this._config.deployment.codesConverter !== undefined &&
            this._config.deployment.clientAPICommunication !== undefined);
    };
    DefaultApiConfiguration.prototype.getComponentCode = function (componentName) {
        var component = this.findComponentByName(componentName);
        return Number(component.id);
    };
    DefaultApiConfiguration.prototype.getStateMachineCode = function (componentName, stateMachineName) {
        var component = this.findComponentByName(componentName);
        var stateMachine = this.findStateMachineByName(component, stateMachineName);
        return Number(stateMachine.id);
    };
    DefaultApiConfiguration.prototype.findStateMachineByName = function (component, stateMachineName) {
        var stateMachine = this.findStateMachine(component, function (stm) { return stm.name === stateMachineName; });
        if (!stateMachine) {
            throw new Error("StateMachine '".concat(stateMachineName, "' not found"));
        }
        return stateMachine;
    };
    DefaultApiConfiguration.prototype.findStateMachineByCode = function (component, stateMachineCode) {
        var stateMachine = this.findStateMachine(component, function (stm) { return Number(stm.id) === stateMachineCode; });
        if (!stateMachine) {
            throw new Error("StateMachine '".concat(stateMachineCode, "' not found"));
        }
        return stateMachine;
    };
    DefaultApiConfiguration.prototype.findStateByCode = function (stateMachine, stateCode) {
        var stateContainer = stateMachine.states;
        var statesArray = Array.isArray(stateContainer === null || stateContainer === void 0 ? void 0 : stateContainer.State)
            ? stateContainer.State
            : (stateContainer === null || stateContainer === void 0 ? void 0 : stateContainer.State)
                ? [stateContainer.State]
                : [];
        var result = statesArray.find(function (state) { return Number(state.id) === stateCode; });
        if (!result) {
            throw new Error("State '".concat(stateCode, "' not found"));
        }
        return result;
    };
    DefaultApiConfiguration.prototype.findComponentByName = function (componentName) {
        var result = this.findComponent(function (component) { return component.name === componentName; });
        if (!result) {
            throw new Error("Component '".concat(componentName, "' not found"));
        }
        return result;
    };
    DefaultApiConfiguration.prototype.findComponentByCode = function (componentCode) {
        var result = this.findComponent(function (component) { return Number(component.id) === componentCode; });
        if (!result) {
            throw new Error("Component '".concat(componentCode, "' not found"));
        }
        return result;
    };
    DefaultApiConfiguration.prototype.findComponent = function (predicate) {
        var codesConverter = this._config.deployment.codesConverter;
        var components = codesConverter.components.component;
        return components.find(predicate);
    };
    DefaultApiConfiguration.prototype.findStateMachine = function (component, predicate) {
        return component.stateMachines.stateMachine.find(predicate);
    };
    DefaultApiConfiguration.prototype.containsComponent = function (componentName) {
        var result = this.findComponent(function (component) { return component.name === componentName; });
        return result ? true : false;
    };
    DefaultApiConfiguration.prototype.containsStateMachine = function (componentName, stateMachineName) {
        var result = this.findComponent(function (component) { return component.name === componentName; });
        if (result) {
            return result.stateMachines.stateMachine.find(function (stm) { return stm.name === stateMachineName; }) != null;
        }
        return false;
    };
    DefaultApiConfiguration.prototype.containsPublisher = function (componentCode, stateMachineCode, messageType) {
        return this.getPublisher(componentCode, stateMachineCode, messageType) !== undefined;
    };
    DefaultApiConfiguration.prototype.containsSubscriber = function (componentCode, stateMachineCode, type) {
        return this.getSubscriber(componentCode, stateMachineCode, type) !== undefined;
    };
    DefaultApiConfiguration.prototype.getPublisherDetails = function (componentCode, stateMachineCode, messageType) {
        var publisher = this.getPublisher(componentCode, stateMachineCode, messageType);
        if (!publisher) {
            throw new Error("publisher not found - component code: ".concat(componentCode, " - statemachine code: ").concat(stateMachineCode, " - message type: ").concat(messageType, " "));
        }
        return {
            eventCode: Number(publisher.attributes.eventCode),
            routingKey: publisher.topic[0].value,
        };
    };
    DefaultApiConfiguration.prototype.getPublisher = function (componentCode, stateMachineCode, messageType) {
        var raw = this._config.deployment.clientAPICommunication.publish;
        var publishArrayRaw = Array.isArray(raw) ? raw : [raw];
        var publishArray = publishArrayRaw.map(normalizeCommunication);
        return publishArray.find(function (pub) {
            return Number(pub.attributes.componentCode) === componentCode &&
                Number(pub.attributes.stateMachineCode) === stateMachineCode &&
                pub.attributes.event === messageType;
        });
    };
    DefaultApiConfiguration.prototype.getSubscriberTopic = function (componentCode, stateMachineCode, type) {
        var subscriber = this.getSubscriber(componentCode, stateMachineCode, type);
        if (!subscriber) {
            throw new Error("Subscriber not found - component code: ".concat(componentCode, " - statemachine code: ").concat(stateMachineCode));
        }
        return subscriber.topic[0].value;
    };
    DefaultApiConfiguration.prototype.getSubscriber = function (componentCode, stateMachineCode, type) {
        var raw = this._config.deployment.clientAPICommunication.subscribe;
        var subscribeArrayRaw = Array.isArray(raw) ? raw : [raw];
        var subscribeArray = subscribeArrayRaw.map(normalizeCommunication);
        return subscribeArray.find(function (sub) {
            return Number(sub.attributes.componentCode) === componentCode &&
                Number(sub.attributes.stateMachineCode) === stateMachineCode &&
                sub.attributes.eventType === SubscriberEventType[type].toUpperCase();
        });
    };
    DefaultApiConfiguration.prototype.getSnapshotTopic = function (componentCode) {
        var raw = this._config.deployment.clientAPICommunication.snapshot;
        var snapshotArrayRaw = Array.isArray(raw) ? raw : [raw];
        var snapshotArray = snapshotArrayRaw.map(normalizeCommunication);
        var snapshot = snapshotArray.find(function (pub) { return Number(pub.attributes.componentCode) === componentCode; });
        if (!snapshot) {
            throw new Error("Snapshot topic not found - component code: ".concat(componentCode));
        }
        return snapshot.topic[0].value;
    };
    DefaultApiConfiguration.prototype.getStateName = function (componentCode, stateMachineCode, stateCode) {
        var component = this.findComponentByCode(componentCode);
        var stateMachine = this.findStateMachineByCode(component, stateMachineCode);
        var state = this.findStateByCode(stateMachine, stateCode);
        return state.name;
    };
    return DefaultApiConfiguration;
}());
export { DefaultApiConfiguration };
//# sourceMappingURL=apiConfiguration.js.map